@@ -8,7 +8,10 @@ use base 'Exporter';
 use Cwd;
 
 our @EXPORT = ('serve_directory_listing');
-our $VERSION = '0.08';
+our $VERSION = '0.09';
+
+use constant TEXT_403 => 'Forbidden';
+use constant TEXT_404 => 'File not found';
 
 # FIXME: see  @{app->static->paths} for list of public directories
 our $public_dir = "public";
@@ -95,9 +98,9 @@ sub _mk_fileserver {
 	my $file = $self->param('file');
 
 	if (! -r "$local/$file") {
-	    $self->status(403);
+	    $self->render( text => TEXT_403, status => 403 );
 	} elsif (-d "$local/$file") {
-	    $self->status(403);
+	    $self->render( status => 403, text => TEXT_403 );
 	} elsif (open my $fh, '<', "$local/$file") {
 	    my $output = join '', <$fh>;
 	    close $fh;
@@ -115,7 +118,7 @@ sub _mk_fileserver {
 		$self->render( data => $output );
 	    }
 	} else {
-	    $self->status(404);
+	    $self->render( text => TEXT_404, status => 404 );
 	}
     };
 }
@@ -134,6 +137,12 @@ sub _mk_dir_listing {
     };
 }
 
+sub _directory_listing_link {
+    my ($href, $text) = @_;
+    return sprintf '<a class="%s" href="%s">%s</a>',
+	"directory-listing-link", $href, $text;
+}
+
 sub _render_directory {
     my $self = shift;
     my $output;
@@ -164,9 +173,9 @@ sub _render_directory {
 	$self->app->log->error(
 	    "MojoX::DirectoryListing: opendir failed on $actual_dir" );
 	if (-d $actual_dir) {
-	    $self->status(403);
+	    $self->render( text => TEXT_403, status => 403 );
 	} else {
-	    $self->status(404);
+	    $self->render( text => TEXT_404, status => 404 );
 	}
 	return;
     }
@@ -177,6 +186,11 @@ sub _render_directory {
 	my $is_dir = -d "$actual_dir/$_";
 	$size = -1 if $is_dir;
 	my $forbidden = ! -r "$actual_dir/$_";
+
+	# another way this item can be forbidden is if
+	#   * it is a directory
+	#   * that directory is not served
+	
 	+{
 	    name => $_,
 	    is_dir => $is_dir,
@@ -210,7 +224,10 @@ sub _render_directory {
 </head>
 <body class="directory-listing">
 <base href="/" />
-<!-- directory listing by MojoX::DirectoryListing -->
+<!-- directory listing by MojoX::DirectoryListing
+     sort column: $sort_column
+     sort order:  $sort_order
+  -->
 ];
 
     my $header = $self->stash("header") //
@@ -244,8 +261,10 @@ sub _render_directory {
 	    $output .= qq[  <th>&nbsp;</th>\n];
 	} else {
 
+	    my $link = _directory_listing_link(
+		"$virtual_dir?C=$col_code;O=$order_code", $text);
 	    $output .= qq[  <th>
-      <a class="directory-listing-link" href="$virtual_dir?C=$col_code;O=$order_code">$text</a> $sortind
+      $link $sortind
   </th>
 ];
 	}
@@ -272,9 +291,10 @@ sub _render_directory {
 	} else {
 	    my $name = $item->{name};
 	    $name = 'Parent Directory' if $name eq '..';
-	    $output .= sprintf $table_element_template,
-		"name",
-		qq[<a class="directory-listing-link" href="$virtual_dir/$item->{name}">$name</a>];
+	    my $href = "$virtual_dir/$item->{name}";
+	    $href =~ s{^//}{/};
+	    my $link = _directory_listing_link($href, $name);
+	    $output .= sprintf $table_element_template, "name", $link;
 	}
 
 
@@ -410,7 +430,7 @@ MojoX::DirectoryListing - show Apache-style directory listings in your Mojolicio
 
 =head1 VERSION
 
-0.08
+0.09
 
 =head1 SYNOPSIS
 
@@ -700,7 +720,7 @@ L<Plack::App::Directory>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2013-2016 Marty O'Brien.
+Copyright 2013-2017 Marty O'Brien.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the the Artistic License (2.0). You may obtain a
